@@ -1,9 +1,9 @@
 ##############################################################################
 #
-# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
+# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
@@ -12,13 +12,19 @@
 ##############################################################################
 """Support for owned objects
 
-$Id: Owned.py 73418 2007-03-21 11:48:43Z mj $
+$Id: Owned.py 68570 2006-06-10 14:06:56Z andreasjung $
 """
 
 import Globals, urlparse, SpecialUsers, ExtensionClass
+from Globals import InitializeClass
+from AccessControl import ClassSecurityInfo
 from AccessControl import getSecurityManager, Unauthorized
+from AccessControl.Permissions import view_management_screens
+from AccessControl.Permissions import take_ownership
 from Acquisition import aq_get, aq_parent, aq_base
-from requestmethod import postonly
+from zope.interface import implements
+
+from interfaces import IOwned
 
 
 UnownableOwner=[]
@@ -31,13 +37,10 @@ _mark=ownableFilter
 
 class Owned(ExtensionClass.Base):
 
-    __ac_permissions__=(
-        ('View management screens',
-         ('manage_owner', 'owner_info')),
-        ('Take ownership',
-         ('manage_takeOwnership','manage_changeOwnershipType'),
-         ("Owner",)),
-        )
+    implements(IOwned)
+
+    security = ClassSecurityInfo()
+    security.setPermissionDefault(take_ownership, ('Owner',))
 
     manage_options=({'label':  'Ownership',
                      'action': 'manage_owner',
@@ -46,8 +49,10 @@ class Owned(ExtensionClass.Base):
                      },
                    )
 
+    security.declareProtected(view_management_screens, 'manage_owner')
     manage_owner=Globals.DTMLFile('dtml/owner', globals())
 
+    security.declareProtected(view_management_screens, 'owner_info')
     def owner_info(self):
         """Get ownership info for display
         """
@@ -63,7 +68,7 @@ class Owned(ExtensionClass.Base):
            }
         return d
 
-    getOwner__roles__=()
+    security.declarePrivate('getOwner')
     def getOwner(self, info=0,
                  aq_get=aq_get,
                  UnownableOwner=UnownableOwner,
@@ -97,7 +102,7 @@ class Owned(ExtensionClass.Base):
             if user is None: user = SpecialUsers.nobody
         return user
 
-    getOwnerTuple__roles__=()
+    security.declarePrivate('getOwnerTuple')
     def getOwnerTuple(self):
         """Return a tuple, (userdb_path, user_id) for the owner.
 
@@ -107,7 +112,7 @@ class Owned(ExtensionClass.Base):
         """
         return aq_get(self, '_owner', None, 1)
 
-    getWrappedOwner__roles__=()
+    security.declarePrivate('getWrappedOwner')
     def getWrappedOwner(self):
         """Get the owner, modestly wrapped in the user folder.
 
@@ -137,7 +142,7 @@ class Owned(ExtensionClass.Base):
 
         return user.__of__(udb)
 
-    changeOwnership__roles__=()
+    security.declarePrivate('changeOwnership')
     def changeOwnership(self, user, recursive=0):
         """Change the ownership to the given user.
 
@@ -171,6 +176,7 @@ class Owned(ExtensionClass.Base):
         if owner == info: return 0
         return security.checkPermission('Take ownership', self)
 
+    security.declareProtected(take_ownership, 'manage_takeOwnership')
     def manage_takeOwnership(self, REQUEST, RESPONSE, recursive=0):
         """Take ownership (responsibility) for an object.
 
@@ -189,8 +195,8 @@ class Owned(ExtensionClass.Base):
         self.changeOwnership(security.getUser(), recursive)
 
         RESPONSE.redirect(REQUEST['HTTP_REFERER'])
-    manage_takeOwnership = postonly(manage_takeOwnership)
 
+    security.declareProtected(take_ownership, 'manage_changeOwnershipType')
     def manage_changeOwnershipType(self, explicit=1,
                                    RESPONSE=None, REQUEST=None):
         """Change the type (implicit or explicit) of ownership.
@@ -210,7 +216,6 @@ class Owned(ExtensionClass.Base):
                 del self._owner
 
         if RESPONSE is not None: RESPONSE.redirect(REQUEST['HTTP_REFERER'])
-    manage_changeOwnershipType = postonly(manage_changeOwnershipType)
 
     def _deleteOwnershipAfterAdd(self):
 
@@ -268,7 +273,7 @@ class Owned(ExtensionClass.Base):
             except: pass
             if s is None: object._p_deactivate()
 
-Globals.default__class_init__(Owned)
+InitializeClass(Owned)
 
 
 class EmergencyUserCannotOwn(Exception):
