@@ -1,37 +1,45 @@
 ##############################################################################
 #
-# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
+# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
-# FOR A PARTICULAR PURPOSE
+# FOR A PARTICULAR PURPOSE.
 #
 ##############################################################################
-"""Access control package"""
+"""Access control package.
 
-__version__='$Revision: 1.176.14.9 $'[11:-2]
+$Id: User.py 40300 2005-11-21 16:54:03Z efge $
+"""
 
-import Globals, socket, SpecialUsers,re
 import os
-from Globals import DTMLFile, MessageDialog, Persistent, PersistentMapping
-from App.Management import Navigation, Tabs
-from Acquisition import Implicit
-from OFS.SimpleItem import Item
+import re
+import socket
 from base64 import decodestring
-from App.ImageFile import ImageFile
-from Role import RoleManager, DEFAULTMAXLISTUSERS
-from PermissionRole import _what_not_even_god_should_do, rolesForPermissionOn
-import AuthEncoding
-from AccessControl import getSecurityManager
-from zExceptions import Unauthorized, BadRequest
-from AccessControl.SecurityManagement import newSecurityManager
-from AccessControl.SecurityManagement import noSecurityManager
-from AccessControl.ZopeSecurityPolicy import _noroles
 
-ListType=type([])
+from Acquisition import Implicit
+from App.Management import Navigation, Tabs
+from Globals import DTMLFile, MessageDialog, Persistent, PersistentMapping
+from Globals import InitializeClass
+from AccessControl import ClassSecurityInfo
+from AccessControl.Permissions import manage_users as ManageUsers
+from OFS.SimpleItem import Item
+from zExceptions import Unauthorized, BadRequest
+from zope.interface import implements
+
+import AuthEncoding
+import SpecialUsers
+from interfaces import IStandardUserFolder
+from PermissionRole import _what_not_even_god_should_do, rolesForPermissionOn
+from Role import RoleManager, DEFAULTMAXLISTUSERS
+from SecurityManagement import getSecurityManager
+from SecurityManagement import newSecurityManager
+from SecurityManagement import noSecurityManager
+from ZopeSecurityPolicy import _noroles
+
 
 _marker=[]
 
@@ -249,7 +257,7 @@ class BasicUser(Implicit):
 
     def has_role(self, roles, object=None):
         """Check to see if a user has a given role or roles."""
-        if type(roles)==type('s'):
+        if isinstance(roles, str):
             roles=[roles]
         if object is not None:
             user_roles = self.getRolesInContext(object)
@@ -264,13 +272,14 @@ class BasicUser(Implicit):
     def has_permission(self, permission, object):
         """Check to see if a user has a given permission on an object."""
         roles=rolesForPermissionOn(permission, object)
-        if type(roles) is type(''):
+        if isinstance(roles, str):
             roles=[roles]
         return self.allowed(object, roles)
 
     def __len__(self): return 1
     def __str__(self): return self.getUserName()
-    __repr__=__str__
+    def __repr__(self):
+        return '<%s %r>' % (self.__class__.__name__, self.getUserName())
 
 
 class SimpleUser(BasicUser):
@@ -450,7 +459,9 @@ class BasicUserFolder(Implicit, Persistent, Navigation, Tabs, RoleManager,
     isAUserFolder=1
     maxlistusers = DEFAULTMAXLISTUSERS
 
-    encrypt_passwords = 0
+    encrypt_passwords = 1
+
+    security = ClassSecurityInfo()
 
     manage_options=(
         (
@@ -463,32 +474,26 @@ class BasicUserFolder(Implicit, Persistent, Navigation, Tabs, RoleManager,
         +Item.manage_options
         )
 
-    __ac_permissions__=(
-        ('Manage users',
-         ('manage_users','getUserNames', 'getUser', 'getUsers',
-          'getUserById', 'user_names', 'setDomainAuthenticationMode',
-          'userFolderAddUser', 'userFolderEditUser', 'userFolderDelUsers',
-          )
-         ),
-        )
-
-
     # ----------------------------------
     # Public UserFolder object interface
     # ----------------------------------
 
+    security.declareProtected(ManageUsers, 'getUserNames')
     def getUserNames(self):
         """Return a list of usernames"""
         raise NotImplementedError
 
+    security.declareProtected(ManageUsers, 'getUsers')
     def getUsers(self):
         """Return a list of user objects"""
         raise NotImplementedError
 
+    security.declareProtected(ManageUsers, 'getUser')
     def getUser(self, name):
         """Return the named user object or None"""
         raise NotImplementedError
 
+    security.declareProtected(ManageUsers, 'getUserById')
     def getUserById(self, id, default=None):
         """Return the user corresponding to the given id.
         """
@@ -527,6 +532,8 @@ class BasicUserFolder(Implicit, Persistent, Navigation, Tabs, RoleManager,
     # Authors of custom user folders don't need to do anything special to
     # support these - they will just call the appropriate '_' methods that
     # user folder subclasses already implement.
+
+    security.declareProtected(ManageUsers, 'userFolderAddUser')
     def userFolderAddUser(self, name, password, roles, domains, **kw):
         """API method for creating a new user object. Note that not all
            user folder implementations support dynamic creation of user
@@ -535,6 +542,7 @@ class BasicUserFolder(Implicit, Persistent, Navigation, Tabs, RoleManager,
             return self._doAddUser(name, password, roles, domains, **kw)
         raise NotImplementedError
 
+    security.declareProtected(ManageUsers, 'userFolderEditUser')
     def userFolderEditUser(self, name, password, roles, domains, **kw):
         """API method for changing user object attributes. Note that not
            all user folder implementations support changing of user object
@@ -543,6 +551,7 @@ class BasicUserFolder(Implicit, Persistent, Navigation, Tabs, RoleManager,
             return self._doChangeUser(name, password, roles, domains, **kw)
         raise NotImplementedError
 
+    security.declareProtected(ManageUsers, 'userFolderDelUsers')
     def userFolderDelUsers(self, names):
         """API method for deleting one or more user objects. Note that not
            all user folder implementations support deletion of user objects."""
@@ -922,6 +931,7 @@ class BasicUserFolder(Implicit, Persistent, Navigation, Tabs, RoleManager,
         self._doDelUsers(names)
         if REQUEST: return self._mainUser(self, REQUEST)
 
+    security.declareProtected(ManageUsers, 'manage_users')
     def manage_users(self,submit=None,REQUEST=None,RESPONSE=None):
         """This method handles operations on users for the web based forms
            of the ZMI. Application code (code that is outside of the forms
@@ -961,6 +971,7 @@ class BasicUserFolder(Implicit, Persistent, Navigation, Tabs, RoleManager,
 
         return self._mainUser(self, REQUEST)
 
+    security.declareProtected(ManageUsers, 'user_names')
     def user_names(self):
         return self.getUserNames()
 
@@ -978,7 +989,7 @@ class BasicUserFolder(Implicit, Persistent, Navigation, Tabs, RoleManager,
 
     def _setId(self, id):
         if id != self.id:
-            raise Globals.MessageDialog(
+            raise MessageDialog(
                 title='Invalid Id',
                 message='Cannot change the id of a UserFolder',
                 action ='./manage_main',)
@@ -987,6 +998,7 @@ class BasicUserFolder(Implicit, Persistent, Navigation, Tabs, RoleManager,
     # Domain authentication support. This is a good candidate to
     # become deprecated in future Zope versions.
 
+    security.declareProtected(ManageUsers, 'setDomainAuthenticationMode')
     def setDomainAuthenticationMode(self, domain_auth_mode):
         """Set the domain-based authentication mode. By default, this
            mode is off due to the high overhead of the operation that
@@ -1001,7 +1013,9 @@ class BasicUserFolder(Implicit, Persistent, Navigation, Tabs, RoleManager,
         """ returns true if domain auth mode is set to true"""
         return getattr(self, '_domain_auth_mode', None)
 
+
 class UserFolder(BasicUserFolder):
+
     """Standard UserFolder object
 
     A UserFolder holds User objects which contain information
@@ -1009,6 +1023,8 @@ class UserFolder(BasicUserFolder):
     UserFolders function chiefly to control access by authenticating
     users and binding them to a collection of roles."""
 
+    implements(IStandardUserFolder)
+
     meta_type='User Folder'
     id       ='acl_users'
     title    ='User Folder'
@@ -1045,7 +1061,7 @@ class UserFolder(BasicUserFolder):
     def _doAddUser(self, name, password, roles, domains, **kw):
         """Create a new user"""
         if password is not None and self.encrypt_passwords \
-                   and not self._isPasswordEncrypted(password):
+                                and not self._isPasswordEncrypted(password):
             password = self._encryptPassword(password)
         self.data[name]=User(name,password,roles,domains)
 
@@ -1087,11 +1103,7 @@ class UserFolder(BasicUserFolder):
                     pass
 
 
-Globals.default__class_init__(UserFolder)
-
-
-
-
+InitializeClass(UserFolder)
 
 
 def manage_addUserFolder(self,dtself=None,REQUEST=None,**ignored):
@@ -1122,7 +1134,6 @@ addr_match=re.compile(r'((\d{1,3}\.){1,3}\*)|((\d{1,3}\.){3}\d{1,3})').match
 host_match=re.compile(r'(([\_0-9a-zA-Z\-]*\.)*[0-9a-zA-Z\-]*)').match
 
 
-
 def domainSpecMatch(spec, request):
     host=''
     addr=''
