@@ -13,17 +13,28 @@
 ##############################################################################
 """Initialize the Five product
 
-$Id: __init__.py 12915 2005-05-31 10:23:19Z philikon $
+$Id: __init__.py 67820 2006-05-02 09:21:45Z philikon $
 """
 import Acquisition
 from Globals import INSTANCE_HOME
 
-import zcml
+from Products.Five import zcml
 
 # public API provided by Five
 # usage: from Products.Five import <something>
-from browser import BrowserView
-from skin import StandardMacros
+from Products.Five.browser import BrowserView
+from Products.Five.skin.standardmacros import StandardMacros
 
+# load the site's ZCML tree (usually site.zcml) upon product
+# initialization
 def initialize(context):
     zcml.load_site()
+
+# some convenience methods/decorators
+
+def fivemethod(func):
+    func.__five_method__ = True
+    return func
+
+def isFiveMethod(m):
+    return hasattr(m, '__five_method__')
