@@ -1,9 +1,9 @@
 ##############################################################################
 #
-# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
+# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
@@ -12,14 +12,14 @@
 ##############################################################################
 """ ZCatalog product
 
-$Id: ZCatalog.py 71167 2006-11-17 19:51:12Z lra $
+$Id: ZCatalog.py 69102 2006-07-12 13:10:28Z shh $
 """
 
 from warnings import warn
 import urllib, time, sys, string, logging
 
 from Globals import DTMLFile, MessageDialog
-import Globals
+from Globals import InitializeClass
 from OFS.Folder import Folder
 from OFS.ObjectManager import ObjectManager
 from DateTime import DateTime
@@ -27,6 +27,7 @@ from Acquisition import Implicit
 from Persistence import Persistent
 from DocumentTemplate.DT_Util import InstanceDict, TemplateDict
 from DocumentTemplate.DT_Util import Eval
+from AccessControl import ClassSecurityInfo
 from AccessControl.Permission import name_trans
 from AccessControl.DTML import RestrictedDTML
 from AccessControl.Permissions import \
@@ -36,8 +37,6 @@ import transaction
 from Products.PluginIndexes.common.PluggableIndex \
      import PluggableIndexInterface
 from Products.PluginIndexes.interfaces import IPluggableIndex
-from Products.PluginIndexes.TextIndex import Splitter
-from zLOG import LOG
 from zope.interface import implements
 
 from Catalog import Catalog, CatalogError
@@ -86,6 +85,11 @@ class ZCatalog(Folder, Persistent, Implicit):
     __implements__ = z2IZCatalog
     implements(z3IZCatalog)
 
+    security = ClassSecurityInfo()
+    security.setPermissionDefault(manage_zcatalog_entries, ('Manager',))
+    security.setPermissionDefault(manage_zcatalog_indexes, ('Manager',))
+    security.setPermissionDefault(search_zcatalog, ('Anonymous', 'Manager'))
+
     meta_type = "ZCatalog"
     icon='misc_/ZCatalog/ZCatalog.gif'
 
@@ -122,46 +126,30 @@ class ZCatalog(Folder, Persistent, Implicit):
          'help': ('OFSP','Ownership.stx'),}
         )
 
-    __ac_permissions__=(
-
-        (manage_zcatalog_entries,
-         ['manage_catalogObject', 'manage_uncatalogObject',
-          'catalog_object', 'uncatalog_object', 'refreshCatalog',
-
-          'manage_catalogView', 'manage_catalogFind',
-          'manage_catalogSchema', 'manage_catalogIndexes',
-          'manage_catalogAdvanced', 'manage_objectInformation',
-
-          'manage_catalogReindex', 'manage_catalogFoundItems',
-          'manage_catalogClear', 'manage_addColumn', 'manage_delColumn',
-          'manage_addIndex', 'manage_delIndex', 'manage_clearIndex',
-          'manage_reindexIndex', 'manage_main', 'availableSplitters',
-          'manage_setProgress',
-
-          # these two are deprecated:
-          'manage_delColumns', 'manage_deleteIndex'
-          ],
-         ['Manager']),
-
-        (search_zcatalog,
-         ['searchResults', '__call__', 'uniqueValuesFor',
-          'getpath', 'schema', 'indexes', 'index_objects',
-          'all_meta_types', 'valid_roles', 'resolve_url',
-          'getobject', 'search'],
-         ['Anonymous', 'Manager']),
-
-        (manage_zcatalog_indexes,
-         ['getIndexObjects'],
-         ['Manager']),
-        )
+    security.declareProtected(manage_zcatalog_entries, 'manage_main')
 
+    security.declareProtected(search_zcatalog, 'all_meta_types')
 
     manage_catalogAddRowForm = DTMLFile('dtml/catalogAddRowForm', globals())
+
+    security.declareProtected(manage_zcatalog_entries, 'manage_catalogView')
     manage_catalogView = DTMLFile('dtml/catalogView',globals())
+
+    security.declareProtected(manage_zcatalog_entries, 'manage_catalogFind')
     manage_catalogFind = DTMLFile('dtml/catalogFind',globals())
+
+    security.declareProtected(manage_zcatalog_entries, 'manage_catalogSchema')
     manage_catalogSchema = DTMLFile('dtml/catalogSchema', globals())
+
+    security.declareProtected(manage_zcatalog_entries, 'manage_catalogIndexes')
     manage_catalogIndexes = DTMLFile('dtml/catalogIndexes', globals())
+
+    security.declareProtected(manage_zcatalog_entries,
+                              'manage_catalogAdvanced')
     manage_catalogAdvanced = DTMLFile('dtml/catalogAdvanced', globals())
+
+    security.declareProtected(manage_zcatalog_entries,
+                              'manage_objectInformation')
     manage_objectInformation = DTMLFile('dtml/catalogObjectInformation',
                                         globals())
 
@@ -224,6 +212,7 @@ class ZCatalog(Folder, Persistent, Implicit):
             URL1 +
             '/manage_catalogAdvanced?manage_tabs_message=Catalog%20Changed')
 
+    security.declareProtected(manage_zcatalog_entries, 'manage_catalogObject')
     def manage_catalogObject(self, REQUEST, RESPONSE, URL1, urls=None):
         """ index Zope object(s) that 'urls' point to """
         if urls:
@@ -232,7 +221,7 @@ class ZCatalog(Folder, Persistent, Implicit):
 
             for url in urls:
                 obj = self.resolve_path(url)
-                if obj is None:
+                if not obj and hasattr(self, 'REQUEST'):
                     obj = self.resolve_url(url, REQUEST)
                 if obj is not None:
                     self.catalog_object(obj, url)
@@ -242,6 +231,8 @@ class ZCatalog(Folder, Persistent, Implicit):
             '/manage_catalogView?manage_tabs_message=Object%20Cataloged')
 
 
+    security.declareProtected(manage_zcatalog_entries,
+                              'manage_uncatalogObject')
     def manage_uncatalogObject(self, REQUEST, RESPONSE, URL1, urls=None):
         """ removes Zope object(s) 'urls' from catalog """
 
@@ -257,6 +248,7 @@ class ZCatalog(Folder, Persistent, Implicit):
             '/manage_catalogView?manage_tabs_message=Object%20Uncataloged')
 
 
+    security.declareProtected(manage_zcatalog_entries, 'manage_catalogReindex')
     def manage_catalogReindex(self, REQUEST, RESPONSE, URL1):
         """ clear the catalog, then re-index everything """
 
@@ -278,6 +270,7 @@ class ZCatalog(Folder, Persistent, Implicit):
                          'Total CPU time: %s' % (`elapse`, `c_elapse`)))
 
 
+    security.declareProtected(manage_zcatalog_entries, 'refreshCatalog')
     def refreshCatalog(self, clear=0, pghandler=None):
         """ re-index everything we can find """
 
@@ -296,8 +289,7 @@ class ZCatalog(Folder, Persistent, Implicit):
 
             p = paths[i]
             obj = self.resolve_path(p)
-            
-            if obj is None and hasattr(self, 'REQUEST'):
+            if not obj:
                 obj = self.resolve_url(p, self.REQUEST)
             if obj is not None:
                 try:
@@ -310,6 +302,7 @@ class ZCatalog(Folder, Persistent, Implicit):
 
         if pghandler: pghandler.finish()
 
+    security.declareProtected(manage_zcatalog_entries, 'manage_catalogClear')
     def manage_catalogClear(self, REQUEST=None, RESPONSE=None, URL1=None):
         """ clears the whole enchilada """
         self._catalog.clear()
@@ -320,6 +313,8 @@ class ZCatalog(Folder, Persistent, Implicit):
               '/manage_catalogAdvanced?manage_tabs_message=Catalog%20Cleared')
 
 
+    security.declareProtected(manage_zcatalog_entries,
+                              'manage_catalogFoundItems')
     def manage_catalogFoundItems(self, REQUEST, RESPONSE, URL2, URL1,
                                  obj_metatypes=None,
                                  obj_ids=None, obj_searchterm=None,
@@ -365,6 +360,7 @@ class ZCatalog(Folder, Persistent, Implicit):
             )
 
 
+    security.declareProtected(manage_zcatalog_entries, 'manage_addColumn')
     def manage_addColumn(self, name, REQUEST=None, RESPONSE=None, URL1=None):
         """ add a column """
         self.addColumn(name)
@@ -375,24 +371,7 @@ class ZCatalog(Folder, Persistent, Implicit):
                 '/manage_catalogSchema?manage_tabs_message=Column%20Added')
 
 
-    def manage_delColumns(self, names, REQUEST=None, RESPONSE=None, URL1=None):
-        """ Deprecated method. Use manage_delColumn instead. """
-        # log a deprecation warning
-        import warnings
-        warnings.warn(
-            "The manage_delColumns method of ZCatalog is deprecated"
-            "since Zope 2.4.2.\n"
-            "This method is only kept for backwards compatibility "
-            "for a while\n"
-            "and will go away in a future release.\n"
-            "\n"
-            "Please use instead the manage_delColumn method.\n"
-            ,DeprecationWarning)
-
-        self.manage_delColumn(names, REQUEST=REQUEST, RESPONSE=RESPONSE,
-                              URL1=URL1)
-
-
+    security.declareProtected(manage_zcatalog_entries, 'manage_delColumn')
     def manage_delColumn(self, names, REQUEST=None, RESPONSE=None, URL1=None):
         """ delete a column or some columns """
         if isinstance(names, str):
@@ -407,6 +386,7 @@ class ZCatalog(Folder, Persistent, Implicit):
                 '/manage_catalogSchema?manage_tabs_message=Column%20Deleted')
 
 
+    security.declareProtected(manage_zcatalog_entries, 'manage_addIndex')
     def manage_addIndex(self, name, type, extra=None,
                         REQUEST=None, RESPONSE=None, URL1=None):
         """add an index """
@@ -418,25 +398,7 @@ class ZCatalog(Folder, Persistent, Implicit):
                 '/manage_catalogIndexes?manage_tabs_message=Index%20Added')
 
 
-    def manage_deleteIndex(self, ids=None, REQUEST=None, RESPONSE=None,
-        URL1=None):
-        """ Deprecated method. Use manage_delIndex instead. """
-        # log a deprecation warning
-        import warnings
-        warnings.warn(
-            "The manage_deleteIndex method of ZCatalog is deprecated"
-            "since Zope 2.4.2.\n"
-            "This method is only kept for backwards compatibility for a "
-            "while\n"
-            "and will go away in a future release.\n"
-            "\n"
-            "Please use instead the manage_delIndex method.\n"
-            ,DeprecationWarning)
-
-        self.manage_delIndex(ids=ids, REQUEST=REQUEST, RESPONSE=RESPONSE,
-                             URL1=URL1)
-
-
+    security.declareProtected(manage_zcatalog_entries, 'manage_delIndex')
     def manage_delIndex(self, ids=None, REQUEST=None, RESPONSE=None,
         URL1=None):
         """ delete an index or some indexes """
@@ -457,6 +419,7 @@ class ZCatalog(Folder, Persistent, Implicit):
                 '/manage_catalogIndexes?manage_tabs_message=Index%20Deleted')
 
 
+    security.declareProtected(manage_zcatalog_entries, 'manage_clearIndex')
     def manage_clearIndex(self, ids=None, REQUEST=None, RESPONSE=None,
         URL1=None):
         """ clear an index or some indexes """
@@ -482,11 +445,10 @@ class ZCatalog(Folder, Persistent, Implicit):
             name = (name,)
 
         paths = self._catalog.uids.keys()
-        num_paths = len(paths)   # inefficient        
 
         i = 0
         if pghandler:
-            pghandler.init('reindexing %s' % name, num_paths)
+            pghandler.init('reindexing %s' % name, len(paths))
 
         for p in paths:
             i+=1
@@ -525,6 +487,7 @@ class ZCatalog(Folder, Persistent, Implicit):
         if pghandler:
             pghandler.finish()
 
+    security.declareProtected(manage_zcatalog_entries, 'manage_reindexIndex')
     def manage_reindexIndex(self, ids=None, REQUEST=None, RESPONSE=None,
                             URL1=None):
         """Reindex indexe(s) from a ZCatalog"""
@@ -544,11 +507,16 @@ class ZCatalog(Folder, Persistent, Implicit):
                 '?manage_tabs_message=Reindexing%20Performed')
 
 
+    # BBB: will be removed in Zope 2.12 (like TextIndex itself)
+    security.declareProtected(manage_zcatalog_entries, 'availableSplitters')
     def availableSplitters(self):
         """ splitter we can add """
+        # This import will trigger a deprecation warning about TextIndex
+        from Products.PluginIndexes.TextIndex import Splitter
         return Splitter.availableSplitters
 
 
+    security.declareProtected(manage_zcatalog_entries, 'catalog_object')
     def catalog_object(self, obj, uid=None, idxs=None, update_metadata=1, pghandler=None):
         """ wrapper around catalog """
 
@@ -594,14 +562,17 @@ class ZCatalog(Folder, Persistent, Implicit):
                 if pghandler:
                     pghandler.info('commiting subtransaction')
 
+    security.declareProtected(manage_zcatalog_entries, 'uncatalog_object')
     def uncatalog_object(self, uid):
         """Wrapper around catalog """
         self._catalog.uncatalogObject(uid)
 
+    security.declareProtected(search_zcatalog, 'uniqueValuesFor')
     def uniqueValuesFor(self, name):
         """Return the unique values for a given FieldIndex """
         return self._catalog.uniqueValuesFor(name)
 
+    security.declareProtected(search_zcatalog, 'getpath')
     def getpath(self, rid):
         """Return the path to a cataloged object given a 'data_record_id_'
         """
@@ -612,10 +583,16 @@ class ZCatalog(Folder, Persistent, Implicit):
         """
         return self._catalog.uids.get(path, default)
 
+    security.declareProtected(search_zcatalog, 'getobject')
     def getobject(self, rid, REQUEST=None):
         """Return a cataloged object given a 'data_record_id_'
         """
-        return self.aq_parent.unrestrictedTraverse(self.getpath(rid))
+        obj = self.aq_parent.unrestrictedTraverse(self.getpath(rid))
+        if not obj:
+            if REQUEST is None:
+                REQUEST=self.REQUEST
+            obj = self.resolve_url(self.getpath(rid), REQUEST)
+        return obj
 
     def getMetadataForUID(self, uid):
         """return the correct metadata given the uid, usually the path"""
@@ -635,17 +612,21 @@ class ZCatalog(Folder, Persistent, Implicit):
         """return the current index contents for the specific rid"""
         return self._catalog.getIndexDataForRID(rid)
 
+    security.declareProtected(search_zcatalog, 'schema')
     def schema(self):
         return self._catalog.schema.keys()
 
+    security.declareProtected(search_zcatalog, 'indexes')
     def indexes(self):
         return self._catalog.indexes.keys()
 
+    security.declareProtected(search_zcatalog, 'index_objects')
     def index_objects(self):
         # This method returns unwrapped indexes!
         # You should probably use getIndexObjects instead
         return self._catalog.indexes.values()
 
+    security.declareProtected(manage_zcatalog_indexes, 'getIndexObjects')
     def getIndexObjects(self):
         # Return a list of wrapped(!) indexes
         getIndex = self._catalog.getIndex
@@ -673,6 +654,7 @@ class ZCatalog(Folder, Persistent, Implicit):
                   'width': 8})
         return r
 
+    security.declareProtected(search_zcatalog, 'searchResults')
     def searchResults(self, REQUEST=None, used=None, **kw):
         """Search the catalog
 
@@ -684,8 +666,10 @@ class ZCatalog(Folder, Persistent, Implicit):
 
         return self._catalog.searchResults(REQUEST, used, **kw)
 
+    security.declareProtected(search_zcatalog, '__call__')
     __call__=searchResults
 
+    security.declareProtected(search_zcatalog, 'search')
     def search(
         self, query_request, sort_index=None, reverse=0, limit=None, merge=1):
         """Programmatic search interface, use for searching the catalog from
@@ -716,6 +700,7 @@ class ZCatalog(Folder, Persistent, Implicit):
     #        except AttributeError:  pass
     #    return self.meta_types+Products.meta_types+pmt
 
+    security.declareProtected(search_zcatalog, 'valid_roles')
     def valid_roles(self):
         "Return list of valid roles"
         obj=self
@@ -834,6 +819,7 @@ class ZCatalog(Folder, Persistent, Implicit):
 
         return result
 
+    security.declareProtected(search_zcatalog, 'resolve_url')
     def resolve_url(self, path, REQUEST):
         """
         Attempt to resolve a url into an object in the Zope
@@ -898,6 +884,7 @@ class ZCatalog(Folder, Persistent, Implicit):
                   '%s unchanged.' % (len(fixed), len(removed), unchanged),
           action='./manage_main')
 
+    security.declareProtected(manage_zcatalog_entries, 'manage_setProgress')
     def manage_setProgress(self, pgthreshold=0, RESPONSE=None, URL1=None):
         """Set parameter to perform logging of reindexing operations very 
            'pgthreshold' objects
@@ -922,8 +909,6 @@ class ZCatalog(Folder, Persistent, Implicit):
         """
 
         LOG.info('Start migration of indexes for %s' % self.absolute_url(1))
-        
-        reindex_ids = []
 
         for idx in self.Indexes.objectValues():
             bases = [str(name) for name in idx.__class__.__bases__]
@@ -958,11 +943,8 @@ class ZCatalog(Folder, Persistent, Implicit):
                 if idx.meta_type == 'DateRangeIndex':
                     setattr(new_idx, '_since_field',  since_field)
                     setattr(new_idx, '_until_field', until_field)
-                reindex_ids.append(idx_id)
-        
-        if reindex_ids:
-            LOG.info('Reindexing %s' % ', '.join(reindex_ids))
-            self.manage_reindexIndex(reindex_ids, REQUEST)
+
+                self.manage_reindexIndex(idx_id, REQUEST)
 
         self._migrated_280 = True
         LOG.info('Finished migration of indexes for %s' % self.absolute_url(1))
@@ -1027,7 +1009,7 @@ class ZCatalog(Folder, Persistent, Implicit):
         return self._catalog.delColumn(name)
 
 
-Globals.default__class_init__(ZCatalog)
+InitializeClass(ZCatalog)
 
 
 def p_name(name):
