@@ -12,8 +12,8 @@
 ##############################################################################
 '''CGI Response Output formatter
 
-$Id: HTTPResponse.py,v 1.75.2.11 2005/10/09 23:53:57 jens Exp $'''
-__version__ = '$Revision: 1.75.2.11 $'[11:-2]
+$Id: HTTPResponse.py 68780 2006-06-21 16:50:26Z tseaver $'''
+__version__ = '$Revision: 1.81 $'[11:-2]
 
 import types, os, sys, re
 import zlib, struct
@@ -27,7 +27,7 @@ from cgi import escape
 nl2sp = maketrans('\n',' ')
 
 # This may get overwritten during configuration
-default_encoding = 'iso-8859-15' 
+default_encoding = 'iso-8859-15'
 
 # Enable APPEND_TRACEBACKS to make Zope append tracebacks like it used to,
 # but a better solution is to make standard_error_message display error_tb.
@@ -97,6 +97,7 @@ for name in en:
 status_codes['nameerror'] = 503
 status_codes['keyerror'] = 503
 status_codes['redirect'] = 300
+status_codes['resourcelockederror'] = 423
 
 
 start_of_header_search = re.compile('(<head[^>]*>)', re.IGNORECASE).search
@@ -148,6 +149,7 @@ class HTTPResponse(BaseResponse):
     realm = 'Zope'
     _error_format = 'text/html'
     _locked_status = 0
+    _locked_body = 0
 
     # Indicate if setBody should content-compress output.
     # 0 - no compression
@@ -201,7 +203,7 @@ class HTTPResponse(BaseResponse):
         """Returns true if this request requested a server shutdown."""
         return self._shutdown_flag is not None
 
-    def setStatus(self, status, reason=None):
+    def setStatus(self, status, reason=None, lock=None):
         '''\
         Sets the HTTP status code of the response; the argument may
         either be an integer or a string from { OK, Created, Accepted,
@@ -236,6 +238,9 @@ class HTTPResponse(BaseResponse):
                 reason = 'Unknown'
         self.setHeader('Status', "%d %s" % (status,str(reason)))
         self.errmsg = reason
+        # lock the status if we're told to
+        if lock:
+             self._locked_status = 1
 
     def setHeader(self, name, value, literal=0):
         '''\
@@ -270,7 +275,8 @@ class HTTPResponse(BaseResponse):
                 latin1_alias_match=re.compile(
                 r'text/html(\s*;\s*charset=((latin)|(latin[-_]?1)|'
                 r'(cp1252)|(cp819)|(csISOLatin1)|(IBM819)|(iso-ir-100)|'
-                r'(iso[-_]8859[-_]1(:1987)?)))?$',re.I).match
+                r'(iso[-_]8859[-_]1(:1987)?)))?$',re.I).match,
+                lock=None
                 ):
         '''\
         Set the body of the response
@@ -287,11 +293,17 @@ class HTTPResponse(BaseResponse):
         If is_error is true then the HTML will be formatted as a Zope error
         message instead of a generic HTML page.
         '''
+        # allow locking of the body in the same way as the status
+        if self._locked_body:
+            return
+        elif lock:
+            self._locked_body = 1
+            
         if not body:
             return self
 
         if isinstance(body, tuple) and len(body) == 2:
-            title,body = body
+            title, body = body
 
         if not isinstance(body, str):
             if hasattr(body,'asHTML'):
@@ -322,13 +334,18 @@ class HTTPResponse(BaseResponse):
                 self.body = body
 
 
+        isHTML = self.isHTML(self.body)
         if not self.headers.has_key('content-type'):
-            isHTML = self.isHTML(self.body)
             if isHTML:
-                c = 'text/html'
+                c = 'text/html; charset=%s' % default_encoding
             else:
-                c = 'text/plain'
+                c = 'text/plain; charset=%s' % default_encoding
             self.setHeader('content-type', c)
+        else:
+            c = self.headers['content-type']
+            if not 'charset=' in  c:
+                c = '%s; charset=%s' % (c, default_encoding)                
+                self.setHeader('content-type', c)
 
         # Some browsers interpret certain characters in Latin 1 as html
         # special characters. These cannot be removed by html_quote,
@@ -428,14 +445,36 @@ class HTTPResponse(BaseResponse):
                                               r'charset=([-_0-9a-z]+' +
                                               r')(?:(?:\s*;)|\Z)',
                                               re.IGNORECASE)):
+
+        def fix_xml_preamble(body, encoding):
+            """ fixes the encoding in the XML preamble according
+                to the charset specified in the content-type header.
+            """
+
+            if body.startswith('<?xml'):
+                pos_right = body.find('?>')  # right end of the XML preamble
+                body = ('<?xml version="1.0" encoding="%s" ?>' % encoding) + body[pos_right+2:]
+            return body
+
         # Encode the Unicode data as requested
+
         if self.headers.has_key('content-type'):
             match = charset_re.match(self.headers['content-type'])
             if match:
                 encoding = match.group(1)
-                return body.encode(encoding)
+                body = body.encode(encoding)
+                body = fix_xml_preamble(body, encoding)
+                return body
+            else:
+
+                ct = self.headers['content-type']
+                if ct.startswith('text/') or ct.startswith('application/'):
+                    self.headers['content-type'] = '%s; charset=%s' % (ct, default_encoding)
+
         # Use the default character encoding
-        return body.encode(default_encoding,'replace')
+        body = body.encode(default_encoding,'replace')
+        body = fix_xml_preamble(body, default_encoding)
+        return body
 
     def setBase(self,base):
         """Set the base URL for the returned document.
@@ -588,42 +627,32 @@ class HTTPResponse(BaseResponse):
     def _error_html(self,title,body):
         # XXX could this try to use standard_error_message somehow?
         return ("""\
-<TABLE BORDER="0" WIDTH="100%">
-<TR VALIGN="TOP">
-
-<TD WIDTH="10%" ALIGN="CENTER">
-&nbsp;
-</TD>
-
-<TD WIDTH="90%">
-  <H2>Site Error</H2>
-  <P>An error was encountered while publishing this resource.
-  </P>""" + \
-  """
-  <P><STRONG>%s</STRONG></P>
+  <h2>Site Error</h2>
+  <p>An error was encountered while publishing this resource.
+  </p>
+  <p><strong>%s</strong></p>
 
   %s""" %(title,body) + \
   """
-  <HR NOSHADE>
+  <hr noshade="noshade"/>
 
-  <P>Troubleshooting Suggestions</P>
+  <p>Troubleshooting Suggestions</p>
 
-  <UL>
-  <LI>The URL may be incorrect.</LI>
-  <LI>The parameters passed to this resource may be incorrect.</LI>
-  <LI>A resource that this resource relies on may be
-      encountering an error.</LI>
-  </UL>
+  <ul>
+  <li>The URL may be incorrect.</li>
+  <li>The parameters passed to this resource may be incorrect.</li>
+  <li>A resource that this resource relies on may be
+      encountering an error.</li>
+  </ul>
 
-  <P>For more detailed information about the error, please
-  refer to error log.
-  </P>
+  <p>For more detailed information about the error, please
+  refer to the error log.
+  </p>
 
-  <P>If the error persists please contact the site maintainer.
+  <p>If the error persists please contact the site maintainer.
   Thank you for your patience.
-  </P>
-</TD></TR>
-</TABLE>""")
+  </p>""")
+
 
     def notFoundError(self,entry='Unknown'):
         self.setStatus(404)
@@ -640,7 +669,7 @@ class HTTPResponse(BaseResponse):
         raise NotFound, self._error_html(
             "Debugging Notice",
             "Zope has encountered a problem publishing your object.<p>"
-            "\n%s" % entry)
+            "\n%s</p>" % entry)
 
     def badRequestError(self,name):
         self.setStatus(400)
@@ -654,7 +683,7 @@ class HTTPResponse(BaseResponse):
             "The parameter, <em>%s</em>, " % name +
             "was omitted from the request.<p>" +
             "Make sure to specify all required parameters, " +
-            "and try the request again."
+            "and try the request again.</p>"
             )
 
     def _unauthorized(self):
@@ -667,9 +696,9 @@ class HTTPResponse(BaseResponse):
         m = "<strong>You are not authorized to access this resource.</strong>"
         if self.debug_mode:
             if self._auth:
-                m = m + '<p>\nUsername and password are not correct.'
+                m = m + '<p>\nUsername and password are not correct.</p>'
             else:
-                m = m + '<p>\nNo Authorization header found.'
+                m = m + '<p>\nNo Authorization header found.</p>'
         raise Unauthorized, m
 
     def exception(self, fatal=0, info=None,
@@ -759,7 +788,7 @@ class HTTPResponse(BaseResponse):
         if fatal and t is SystemExit and v.code == 0:
             body = self.setBody(
                 (str(t),
-                 'Zope has exited normally.<p>' + self._traceback(t, v, tb)),
+                 'Zope has exited normally.<p>' + self._traceback(t, v, tb) + '</p>'),
                 is_error=1)
         else:
             try:
@@ -770,7 +799,8 @@ class HTTPResponse(BaseResponse):
                 body = self.setBody(
                     (str(t),
                      'Sorry, a site error occurred.<p>'
-                     + self._traceback(t, v, tb)),
+                     + self._traceback(t, v, tb)
+                     + '</p>'),
                     is_error=1)
             elif self.isHTML(b):
                 # error is an HTML document, not just a snippet of html
